from shlex import quote as shell_quote

__all__ = ("in_bash", "in_conda", "in_virtualenv")


def in_bash(command: str, quote: str = "'", escape: bool = False, login: bool = True) -> str:
    """Run command inside bash.

    Args:
        command (str): string command to run
        quote (str, optional): Optional simple quoting, without escaping. May cause mismatched quote problems. Defaults to "'".
        escape (bool, optional): Full shell escaping. Defaults to False.
        login (bool, optional): Run in login shell (-l). Defaults to True.

    Returns:
        str: String command to run, starts with "bash"
    """
    if escape:
        command = shell_quote(command)
    if quote:
        command = f"{quote}{command}{quote}"
    if login:
        bash_flags = "-lc"
    else:
        bash_flags = "-c"
    return f"bash {bash_flags} {command}"


def in_conda(env: str, command: str, tool: str = "micromamba") -> str:
    return f"{tool} activate {env} && {command}"


def in_virtualenv(env: str, command: str) -> str:
    return f"source {env}/bin/activate && {command}"



from typing import List, Optional

from pydantic import BaseModel, Field

from .common import Tool
from .git import GitRepo

__all__ = ("PipLibrary", "LibraryList")


class PipLibrary(BaseModel):
    name: str

    version_constraint: str = ""
    install_deps: bool = False
    tool: Tool = "pip"

    def install(self):
        tool = "uv pip" if self.tool == "uv" else "pip"
        install_deps_flag = "" if self.install_deps else "--no-deps "
        return f"{tool} install {install_deps_flag}{self.name}{self.version_constraint}"


class LibraryList(BaseModel):
    pip: List[PipLibrary] = Field(default_factory=list)
    git: List[GitRepo] = Field(default_factory=list)

    virtualenv: Optional[str] = ""

    # def install(self):
    #     return "pip install " + " ".join(lib.name for lib in self.libraries)



from airflow.exceptions import AirflowFailException, AirflowSkipException

__all__ = (
    "skip",
    "fail",
    "pass_",
)


def skip():
    raise AirflowSkipException


def fail():
    raise AirflowFailException


def pass_():
    pass



from typing import Literal

from pydantic import BaseModel

Tool = Literal["pip", "uv"]


__all__ = ("BaseInstallable",)


class BaseInstallable(BaseModel):
    name: str
    install_deps: bool = False
    tool: Tool = "pip"
    virtualenv: str = ""
